@@ -13,7 +13,7 @@ from operator import add
 
 from packaging.version import Version
 
-from distributed import Client, SchedulerPlugin, WorkerPlugin, futures_of, wait
+from distributed import Client, SchedulerPlugin, futures_of, wait
 from distributed.utils_test import cleanup  # noqa F401
 from distributed.utils_test import client as c  # noqa F401
 from distributed.utils_test import (  # noqa F401
@@ -23,6 +23,7 @@ from distributed.utils_test import (  # noqa F401
     loop,
     loop_in_thread,
     popen,
+    s,
     varying,
 )
 
@@ -30,9 +31,7 @@ import dask
 import dask.bag as db
 from dask import compute, delayed, persist
 from dask.base import compute_as_if_collection, get_scheduler
-from dask.blockwise import Blockwise
 from dask.delayed import Delayed
-from dask.layers import ShuffleLayer, SimpleShuffleLayer
 from dask.utils import get_named_args, get_scheduler_lock, tmpdir, tmpfile
 from dask.utils_test import inc
 
@@ -72,17 +71,11 @@ def test_can_import_nested_things():
 @gen_cluster(client=True)
 async def test_persist(c, s, a, b):
     x = delayed(inc)(1)
-    (x2,) = persist(x)
+    x2 = c.persist(x)
 
     await wait(x2)
     assert x2.key in a.data or x2.key in b.data
 
-    y = delayed(inc)(10)
-    y2, one = persist(y, 1)
-
-    await wait(y2)
-    assert y2.key in a.data or y2.key in b.data
-
 
 def test_persist_nested(c):
     a = delayed(1) + 5
@@ -113,10 +106,6 @@ def test_futures_to_delayed_dataframe(c):
     ddf = dd.from_delayed(futures)
     dd.utils.assert_eq(ddf.compute(), pd.concat([df, df], axis=0))
 
-    # Make sure from_delayed is Blockwise
-    if not dd._dask_expr_enabled():
-        assert isinstance(ddf.dask.layers[ddf._name], Blockwise)
-
     with pytest.raises(TypeError):
         ddf = dd.from_delayed([1, 2])
 
@@ -133,8 +122,7 @@ def test_from_delayed_dataframe(c):
     dd.utils.assert_eq(ddf, df, scheduler=c)
 
 
-@pytest.mark.parametrize("fuse", [True, False])
-def test_fused_blockwise_dataframe_merge(c, fuse):
+def test_fused_blockwise_dataframe_merge(c):
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
 
@@ -149,12 +137,11 @@ def test_fused_blockwise_dataframe_merge(c, fuse):
     df1 += 10
     df2 += 10
 
-    with dask.config.set({"optimization.fuse.active": fuse}):
-        ddfm = ddf1.merge(ddf2, on=["x"], how="left", shuffle_method="tasks")
-        ddfm.head()  # https://github.com/dask/dask/issues/7178
-        dfm = ddfm.compute().sort_values("x")
-        # We call compute above since `sort_values` is not
-        # supported in `dask.dataframe`
+    ddfm = ddf1.merge(ddf2, on=["x"], how="left", shuffle_method="tasks")
+    ddfm.head()  # https://github.com/dask/dask/issues/7178
+    dfm = ddfm.compute().sort_values("x")
+    # We call compute above since `sort_values` is not
+    # supported in `dask.dataframe`
     dd.utils.assert_eq(
         dfm, df1.merge(df2, on=["x"], how="left").sort_values("x"), check_index=False
     )
@@ -388,7 +375,7 @@ def test_zarr_distributed_with_explicit_directory_store(c, zarr):
         if Version(zarr.__version__) < Version("3.0.0.a0"):
             s = zarr.storage.DirectoryStore(d)
         else:
-            s = zarr.storage.LocalStore(d, mode="a")
+            s = zarr.storage.LocalStore(d, read_only=False)
         z = zarr.open_array(
             shape=a.shape,
             chunks=chunks,
@@ -411,7 +398,7 @@ def test_zarr_distributed_with_explicit_memory_store(c, zarr):
     if Version(zarr.__version__) < Version("3.0.0.a0"):
         s = zarr.storage.MemoryStore()
     else:
-        s = zarr.storage.MemoryStore(mode="a")
+        s = zarr.storage.MemoryStore(read_only=False)
     z = zarr.open_array(
         shape=a.shape,
         chunks=chunks,
@@ -444,7 +431,7 @@ def test_scheduler_equals_client(c):
 @gen_cluster(client=True)
 async def test_await(c, s, a, b):
     x = dask.delayed(inc)(1)
-    x = await x.persist()
+    x = await c.persist(x)
     assert x.key in s.tasks
     assert a.data or b.data
     assert all(f.done() for f in futures_of(x))
@@ -532,18 +519,14 @@ def test_blockwise_array_creation(c, io, fuse):
 @pytest.mark.parametrize(
     "io",
     [
-        "parquet-pyarrow",
-        pytest.param(
-            "parquet-fastparquet", marks=pytest.mark.skip_with_pyarrow_strings
-        ),
+        "parquet",
         "csv",
         # See https://github.com/dask/dask/issues/9793
         pytest.param("hdf", marks=pytest.mark.flaky(reruns=5)),
     ],
 )
-@pytest.mark.parametrize("fuse", [True, False, None])
 @pytest.mark.parametrize("from_futures", [True, False])
-def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
+def test_blockwise_dataframe_io(c, tmpdir, io, from_futures):
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
 
@@ -556,15 +539,10 @@ def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
     else:
         ddf0 = dd.from_pandas(df, npartitions=3)
 
-    if io == "parquet-pyarrow":
+    if io == "parquet":
         pytest.importorskip("pyarrow")
         ddf0.to_parquet(str(tmpdir))
         ddf = dd.read_parquet(str(tmpdir))
-    elif io == "parquet-fastparquet":
-        pytest.importorskip("fastparquet")
-        with pytest.warns(FutureWarning):
-            ddf0.to_parquet(str(tmpdir), engine="fastparquet")
-            ddf = dd.read_parquet(str(tmpdir), engine="fastparquet")
     elif io == "csv":
         ddf0.to_csv(str(tmpdir), index=False)
         ddf = dd.read_csv(os.path.join(str(tmpdir), "*"))
@@ -578,14 +556,27 @@ def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
 
     df = df[["x"]] + 10
     ddf = ddf[["x"]] + 10
-    if not dd._dask_expr_enabled():
-        with dask.config.set({"optimization.fuse.active": fuse}):
-            ddf.compute()
-            dsk = dask.dataframe.optimize(ddf.dask, ddf.__dask_keys__())
-            # dsk should not be a dict unless fuse is explicitly True
-            assert isinstance(dsk, dict) == bool(fuse)
 
-            dd.assert_eq(ddf, df, check_index=False)
+
+@gen_cluster(client=True)
+async def test_client_compute_parquet(c, s, a, b, tmpdir):
+    pd = pytest.importorskip("pandas")
+    dd = pytest.importorskip("dask.dataframe")
+    df = pd.DataFrame({"x": [1, 2, 3] * 5, "y": range(15)})
+    ddf0 = dd.from_pandas(df, npartitions=3)
+    with dask.config.set({"admin.async-client-fallback": "sync"}):
+        with pytest.warns(UserWarning, match="asynchronous"):
+            ddf0.to_parquet(str(tmpdir))
+
+        ddf = dd.read_parquet(str(tmpdir))
+        await c.gather(c.compute(ddf))
+        with pytest.warns(UserWarning, match="asynchronous"):
+            dask.compute(ddf)
+    with dask.config.set({"admin.async-client-fallback": None}):
+        with pytest.raises(RuntimeError, match="asynchronous"):
+            ddf0.to_parquet(str(tmpdir))
+        with pytest.raises(RuntimeError, match="asynchronous"):
+            dask.compute(ddf)
 
 
 def test_blockwise_fusion_after_compute(c):
@@ -737,75 +728,28 @@ async def test_map_partitions_partition_info(c, s, a, b):
     assert res[1] == {"number": 1, "division": 5}
 
 
-@gen_cluster(client=True)
-async def test_futures_in_subgraphs(c, s, a, b):
+def test_futures_in_subgraphs(loop_in_thread):
     """Copied from distributed (tests/test_client.py)"""
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
+    with cluster() as (s, [a, b]), Client(s["address"], loop=loop_in_thread):
+        ddf = dd.from_pandas(
+            pd.DataFrame(
+                dict(
+                    uid=range(50),
+                    enter_time=pd.date_range(
+                        start="2020-01-01", end="2020-09-01", periods=50, tz="UTC"
+                    ),
+                )
+            ),
+            npartitions=5,
+        )
 
-    ddf = dd.from_pandas(
-        pd.DataFrame(
-            dict(
-                uid=range(50),
-                enter_time=pd.date_range(
-                    start="2020-01-01", end="2020-09-01", periods=50, tz="UTC"
-                ),
-            )
-        ),
-        npartitions=1,
-    )
-
-    ddf = ddf[ddf.uid.isin(range(29))].persist()
-    ddf["day"] = ddf.enter_time.dt.day_name()
-    ddf = await c.submit(dd.categorical.categorize, ddf, columns=["day"], index=False)
-
-
-@pytest.mark.parametrize(
-    "max_branch, expected_layer_type",
-    [
-        (32, SimpleShuffleLayer),
-        (2, ShuffleLayer),
-    ],
-)
-@gen_cluster(client=True, nthreads=[("", 1)] * 2)
-async def test_shuffle_priority(c, s, a, b, max_branch, expected_layer_type):
-    pd = pytest.importorskip("pandas")
-    dd = pytest.importorskip("dask.dataframe")
-    if dd._dask_expr_enabled():
-        pytest.skip("Checking layers doesn't make sense")
-
-    class EnsureSplitsRunImmediatelyPlugin(WorkerPlugin):
-        failure = False
-
-        def setup(self, worker):
-            self.worker = worker
-
-        def transition(self, key, start, finish, **kwargs):
-            if finish == "executing" and not all(
-                "split" in ts.key for ts in self.worker.state.executing
-            ):
-                if any("split" in ts.key for ts in list(self.worker.state.ready)):
-                    EnsureSplitsRunImmediatelyPlugin.failure = True
-                    raise RuntimeError("Split tasks are not prioritized")
-
-    await c.register_plugin(EnsureSplitsRunImmediatelyPlugin())
-
-    # Test marked as "flaky" since the scheduling behavior
-    # is not deterministic. Note that the test is still
-    # very likely to fail every time if the "split" tasks
-    # are not prioritized correctly
-
-    df = pd.DataFrame({"a": range(1000)})
-    ddf = dd.from_pandas(df, npartitions=10)
-
-    ddf2 = ddf.shuffle("a", shuffle_method="tasks", max_branch=max_branch)
-
-    shuffle_layers = set(ddf2.dask.layers) - set(ddf.dask.layers)
-    for layer_name in shuffle_layers:
-        if "shuffle" in layer_name:
-            assert isinstance(ddf2.dask.layers[layer_name], expected_layer_type)
-    await c.compute(ddf2)
-    assert not EnsureSplitsRunImmediatelyPlugin.failure
+        ddf = ddf[ddf.uid.isin(range(29))].persist()
+        ddf["local_time"] = ddf.enter_time.dt.tz_convert("US/Central")
+        ddf["day"] = ddf.enter_time.dt.day_name()
+        ddf = ddf.categorize(columns=["day"], index=False)
+        ddf.compute()
 
 
 @gen_cluster(client=True)
@@ -814,10 +758,9 @@ async def test_map_partitions_da_input(c, s, a, b):
     np = pytest.importorskip("numpy")
     pd = pytest.importorskip("pandas")
     da = pytest.importorskip("dask.array")
-    dd = pytest.importorskip("dask.dataframe")
+    pytest.importorskip("dask.dataframe")
     datasets = pytest.importorskip("dask.datasets")
-    if dd._dask_expr_enabled():
-        pytest.skip("roundtripping through arrays doesn't work yet")
+    pytest.xfail("roundtripping through arrays doesn't work yet")
 
     def f(d, a):
         assert isinstance(d, pd.DataFrame)
@@ -836,8 +779,7 @@ def test_map_partitions_df_input():
     """
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
-    if dd._dask_expr_enabled():
-        pytest.skip("map partitions can't deal with delayed properly")
+    pytest.xfail("map partitions can't deal with delayed properly")
 
     def f(d, a):
         assert isinstance(d, pd.DataFrame)
@@ -950,23 +892,10 @@ def test_get_scheduler_without_distributed_raises():
         get_scheduler(scheduler="distributed")
 
 
-def test_get_scheduler_with_distributed_active(c):
-    assert get_scheduler() == c.get
-    warning_message = (
-        "Running on a single-machine scheduler when a distributed client "
-        "is active might lead to unexpected results."
-    )
-    with pytest.warns(UserWarning, match=warning_message) as user_warnings_a:
-        get_scheduler(scheduler="threads")
-        get_scheduler(scheduler="sync")
-    assert len(user_warnings_a) == 2
-
-
 def test_get_scheduler_with_distributed_active_reset_config(c):
     assert get_scheduler() == c.get
     with dask.config.set(scheduler="threads"):
-        with pytest.warns(UserWarning):
-            assert get_scheduler() != c.get
+        assert get_scheduler() != c.get
         with dask.config.set(scheduler=None):
             assert get_scheduler() == c.get
 
@@ -1035,33 +964,33 @@ def test_write_single_hdf(c, lock_param):
         ddf.to_hdf(str(f), key="/ds_*", lock=lock_param)
 
 
-@gen_cluster(config={"scheduler": "sync"}, nthreads=[])
-async def test_get_scheduler_default_client_config_interleaving(s):
+def test_get_scheduler_default_client_config_interleaving(s):
     # This test is using context managers intentionally. We should not refactor
     # this to use it in more places to make the client closing cleaner.
-    with pytest.warns(UserWarning):
+    s_address = s["address"]
+    with dask.config.set(scheduler="sync"):
         assert dask.base.get_scheduler() == dask.local.get_sync
         with dask.config.set(scheduler="threads"):
             assert dask.base.get_scheduler() == dask.threaded.get
-            client = await Client(s.address, set_as_default=False, asynchronous=True)
+            client = Client(s_address, set_as_default=False)
             try:
                 assert dask.base.get_scheduler() == dask.threaded.get
             finally:
-                await client.close()
+                client.close()
 
-            client = await Client(s.address, set_as_default=True, asynchronous=True)
+            client = Client(s_address, set_as_default=True)
             try:
                 assert dask.base.get_scheduler() == client.get
             finally:
-                await client.close()
+                client.close()
             assert dask.base.get_scheduler() == dask.threaded.get
 
             # FIXME: As soon as async with uses as_current this will be true as well
-            # async with Client(s.address, set_as_default=False, asynchronous=True) as c:
+            # async with Client(s_address, set_as_default=False, asynchronous=True) as c:
             #     assert dask.base.get_scheduler() == c.get
             # assert dask.base.get_scheduler() == dask.threaded.get
 
-            client = await Client(s.address, set_as_default=False, asynchronous=True)
+            client = Client(s_address, set_as_default=False)
             try:
                 assert dask.base.get_scheduler() == dask.threaded.get
                 with client.as_current():
@@ -1069,24 +998,24 @@ async def test_get_scheduler_default_client_config_interleaving(s):
                     assert sc == client.get
                 assert dask.base.get_scheduler() == dask.threaded.get
             finally:
-                await client.close()
+                client.close()
 
             # If it comes to a race between default and current, current wins
-            client = await Client(s.address, set_as_default=True, asynchronous=True)
-            client2 = await Client(s.address, set_as_default=False, asynchronous=True)
+            client = Client(s_address, set_as_default=True)
+            client2 = Client(s_address, set_as_default=False)
             try:
                 with client2.as_current():
                     assert dask.base.get_scheduler() == client2.get
                 assert dask.base.get_scheduler() == client.get
             finally:
-                await client.close()
-                await client2.close()
+                client.close()
+                client2.close()
 
             assert dask.base.get_scheduler() == dask.threaded.get
 
         assert dask.base.get_scheduler() == dask.local.get_sync
 
-        client = await Client(s.address, set_as_default=True, asynchronous=True)
+        client = Client(s_address, set_as_default=True)
         try:
             assert dask.base.get_scheduler() == client.get
             with dask.config.set(scheduler="threads"):
@@ -1094,7 +1023,7 @@ async def test_get_scheduler_default_client_config_interleaving(s):
                 with client.as_current():
                     assert dask.base.get_scheduler() == client.get
         finally:
-            await client.close()
+            client.close()
 
 
 @gen_cluster(client=True)
@@ -1112,14 +1041,14 @@ async def test_release_persisted_futures_without_gc(c, s, a, b):
     gc.disable()
     try:
         x = da.arange(100, chunks=(20,))
-        y = (x + 1).persist()
+        y = c.persist(x + 1)
         future_refs = []
         coly = weakref.ref(y)
         future_refs.extend([weakref.ref(fut) for fut in futures_of(y)])
         colz = weakref.ref(y)
         y = await y
         # The issue only occurs if we persist the future again.
-        z = y[:20].persist()
+        z = c.persist(y[:20])
 
         future_refs.extend([weakref.ref(fut) for fut in futures_of(z)])
         colz = weakref.ref(z)
@@ -1132,3 +1061,33 @@ async def test_release_persisted_futures_without_gc(c, s, a, b):
             await asyncio.sleep(0.01)
     finally:
         gc.enable()
+
+
+@gen_cluster(client=True)
+async def test_delayed_future_with_kwargs(c, s, a, b):
+    fut = await c.scatter(1)
+
+    result = dask.delayed(inc)(x=fut)
+    assert await c.compute(result) == 2
+
+
+@gen_cluster(client=True)
+async def test_fusion_barrier_task(c, s, a, b):
+    np = pytest.importorskip("numpy")
+    da = pytest.importorskip("dask.array")
+
+    a = np.random.default_rng().uniform(0, 1, 100).reshape((10, 10))
+    x = da.from_array(a, chunks=(10, 1))
+    new = ((1,) * 10, (10,))
+    x2 = da.rechunk(x, chunks=new, method="p2p")
+    result = await c.compute(x2)
+    da.assert_eq(result, a)
+
+
+@gen_cluster(client=True)
+async def test_delayed_future(c, s, a, b):
+    fut = await c.scatter(1)
+    result = dask.delayed(fut)
+    res = await c.compute(result)
+    assert res == 1
+    assert result.key == fut.key
